/*
  Library support for FlexRadio 6000 Rigs
  Copyright (C)2015 Vincenzo Stefanazzi - IW7DMH. All right reserved

  This library is free software; you can redistribute it and/or
  modify it under the terms of the CC BY-NC-SA 3.0 license.
  https://creativecommons.org/licenses/by-nc-sa/3.0/
  
  The license applies to all part of the library including the 
  examples and tools supplied with the library.
*/
#include "FlexCat.h"
FlexCat::FlexCat(){
}


void FlexCat::process(FlexRig &fRig) {

  //reading response  
  while (Serial3.available()) {
		//Serial.print("numero di byte:");Serial.println(client.available());
		
		char c=Serial3.read();
		//Serial.print(c);	
		
		if (c==13) { 
			//Serial.print("<CR>");
			continue;
		} else if (c==10) {
            //Serial.print("<NL>");	
			continue;
		} else {
			//Serial.print(c);
		}	
		if (c==';') {
        	parseRequest(buffer,fRig);
    		buffer="";
        } else {
    		buffer+=c;		
        }
  }
}


void FlexCat::parseRequest(String r,FlexRig &fRig) {

  /*
  Serial.print("echo:"); Serial.print(r); Serial.println("<");
  Serial.print("length:"); Serial.println(r.length());
  for (int i=0;i<r.length();i++) {
	Serial.print(i);Serial.println(r.substring(i,i+1));
  }
  */
  if (r.equals("FA")) {
    if (!fRig.slice[0].in_use) {
		Serial3.print("?;");
		return;
	}
    char s[13];
	sprintf(s,"FA%011d;",fRig.slice[0].RF_frequency);
    Serial3.print(s);
	//Serial.print(s);
    return;
  }
  
  if (r.equals("ZZFA")) {
    if (!fRig.slice[0].in_use) {
		Serial3.print("?;");
		return;
	}
    char s[16];
	sprintf(s,"ZZFA%011d;",fRig.slice[0].RF_frequency);
    Serial3.print(s);
    return;
  }
  
  if (r.equals("FB")) {
    if (!fRig.slice[1].in_use) {
		Serial3.print("?;");
		return;
	}
    char s[14];
	sprintf(s,"FB%011d;",fRig.slice[1].RF_frequency);
    Serial3.print(s);
	//Serial.print(s);
    return;
  }
  
  if (r.equals("ZZFB")) {
    if (!fRig.slice[1].in_use) {
		Serial3.print("?;");
		return;
	}
    char s[16];
	sprintf(s,"ZZFB%011d;",fRig.slice[0].RF_frequency);
    Serial3.print(s);
    return;
  }
  
  if (r.equals("IF")) {
    /*
								  P P P P  P P P  P  P  P  P  P
		<     P1    > <P2> < P3 > 4 5 6 7  8 9 10 11 12 13 14 15
		IF00014018270 0010 +00000 0 0 0 00 0 3 0  0  0  0  00 0
		IF12345678901 2345 678901 2 3 4 56 7 8 9  0  1  2  34 5 
			Freq      STEP  RXIT  S S 0 00 M M F  0  F  0  00 0 
										   O D R     T  
										   X
	*/
	
    char s[38];
	sprintf(s,"IF%011d%04d%+06d%01d%01d000%01d%01d%01d0%01d0000;",
	          fRig.slice[0].RF_frequency,
			  fRig.slice[0].step,
			  fRig.slice[0].rit_freq,
			  fRig.slice[0].rit_on,
			  fRig.slice[0].xit_on,
			  getMox(fRig.interlock.state),
			  getMd(fRig.slice[0].intMode),
			  getFr(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx),
			  getFt(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx));
    Serial3.print(s);
    //Serial.print(s);
	return;
  }
  
  if (r.equals("ZZIF")) {
    /*
								  P P P P  P P P  P  P  P  P  P
		<     P1    > <P2> < P3 > 4 5 6 7  8 9 10 11 12 13 14 15
		IF00014018270 0010 +00000 0 0 0 00 0 3 0  0  0  0  00 0
		IF12345678901 2345 678901 2 3 4 56 7 8 9  0  1  2  34 5 
			Freq      STEP  RXIT  S S 0 00 M M F  0  F  0  00 0 
										   O D R     T  
										   X
	*/
	
    Serial3.print("ZZIF000140128261000+01234110000040000000;");
	char s[40];
	sprintf(s,"ZZIF%011d%04d%+06d%01d%01d000%01d%02d%01d0%01d0000;",
	          fRig.slice[0].RF_frequency,
			  fRig.slice[0].step,
			  fRig.slice[0].rit_freq,
			  fRig.slice[0].rit_on,
			  fRig.slice[0].xit_on,
			  getMox(fRig.interlock.state),
			  getZzmd(fRig.slice[0].intMode,fRig.transmit.cwl_enabled),
			  getFr(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx),
			  getFt(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx));
    Serial3.print(s);
    return;
  }
  
  
  if (r.equals("FR")) {
    char s[4];
	sprintf(s,"FR%01d;",getFr(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx));
    Serial3.print(s);
    return;
  }
  
  if (r.equals("FT")) {
    char s[4];
	sprintf(s,"FT%01d;",getFt(fRig.slice[0].in_use,fRig.slice[1].in_use,fRig.slice[0].tx,fRig.slice[1].tx));
    Serial3.print(s);
    return;
  }
  
  if (r.equals("AI0")) {
    AI=0;
    return;
  }
  
  if (r.equals("AI1")) {
    AI=1;
    return;
  }
  Serial.println("unhandled cmd!");
}


int FlexCat::getMd(int intMode) {
	int md=0;
	switch (intMode) {
		case USB:
			md=1;
			break;
		case LSB:
			md=2;
			break;
		case CW:
			md=3;
			break;
		case FM:
			md=4;
			break;
		case AM:
			md=5;
			break;
		case DIGL:
			md=6;
			break;
		case DIGU:
			md=9;
			break;
		default:
			md=1;
	}
	return md;
}

int FlexCat::getZzmd(int intMode,int cwlEnabled) {
	int md=0;
	switch (intMode) {
		case LSB:
			md=1;
			break;
		case USB:
			md=2;
			break;
		case CW:
			md=4;
			if (cwlEnabled) md=3;
			break;
		case FM:
			md=5;
			break;
		case AM:
			md=6;
			break;
		case DIGU:
			md=7;
			break;
		case DIGL:
			md=9;
			break;
		case SAM_:   //Be careful SAM is and Arduino reserved word. I used instead SAM_
			md=10;
			break;
		case NFM:
			md=11;
			break;
		case DFM:
			md=12;
			break;
		default:
			md=1;
	}
	return md;
}

void FlexCat::sendFA(int RF_freq){
    if (AI) {
		char s[14];
		sprintf(s,"FA%011d;",RF_freq);
		Serial3.print(s);
	}
}

void FlexCat::sendFB(int RF_freq){
    if (AI) {
		char s[14];
		sprintf(s,"FB%011d;",RF_freq);
		Serial3.print(s);
	}
}

int FlexCat::getMox(String state) {
	int s=0;
	if (state.equals("TRANSMITTING")) s=1;	
	return s;
}

int FlexCat::getFr(int inUseA,int inUseB,int txA,int txB) {
  int fr=0;
  if (inUseB && txA) fr=1;
  return fr;
}

int FlexCat::getFt(int inUseA,int inUseB,int txA,int txB) {
  int ft=0;
  if (inUseB && txB) ft=1;
  return ft;
}